/*
 * ol_avr_uart.c
 * 
 * Version 01.1-11-2018
 * 
 * 
 * Copyright 2018 Gerhard Hinze <gerhard@gerhard-CELSIUS-W370>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of thVersion 01-11-2018e GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA..
 * 
 * 
 */

 /* 
 * Geany Erstellen-Einstellungen für STK500 mit ATmega32
 * 
 * Compil.:  avr-gcc -mmcu=atmega32 -Os %f
 * 
 * Build  :  avr-objcopy -j .text -j .data -O ihex a.out
 * 
 * Flash  :  avrdude -c stk500 -P /dev/ttyS0 -p m32   -U flash:w:a.out
 * 
 * Exec-AVRDude : avrdude -c stk500 -P /dev/ttyS0 -p m32   -U flash:w:a.out
*/

#ifndef F_CPU
#define F_CPU  3686400UL  //STK500
// #define F_CPU  8000000UL  // ATM128 Board
// #define F_CPU  16000000UL // beliebig, auch Arduino
#endif  //F_CPU


/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*====================================================================*/


#include <avr/io.h>
#include <stdio.h>  //printf
#include <util/delay.h>
#include "ol_avr_uart.h"

/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

/*====================================================================*/
/*                                                                    */
/*                 Uart der MCU initalisieren                         */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

void UART_Init()
{ 
  UBRRL = UBRR_VALUE;			//set baud rate
  UBRRH = UBRR_VALUE >> 8;
  UCSRA = 0;				    //no U2X, MPCM
  UCSRC = 1<<URSEL^1<<UCSZ1^1<<UCSZ0;	//8 Bit
  UCSRB = 1<<RXEN^1<<TXEN;		//enable RX, TX
}

/*====================================================================*/
/*                                                                    */
/*             Sende ein einzelnes Zeichen(Char)                      */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

void UART_Write( UCHR c )
{
  while( (UCSRA & 1<<UDRE) == 0 );
  UDR = c;
}

/*====================================================================*/
/*                                                                    */
/*           Sende einen String                                       */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

void UART_Write_Text(UCHR *text)
{
  while (*text)
    {   
		/* so lange *s != '\0' also ungleich dem "String-Endezeichen(Terminator)" */
        UART_Write(*text);
        text++;
    }
  
}

/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*          Sende einen String mit CR/NL                              */
/*                                                                    */
/*====================================================================*/

void UART_Write_Text_NL(UCHR *text)
{
  while (*text)
    {   
		/* so lange *s != '\0' also ungleich dem "String-Endezeichen*/
        UART_Write(*text);
        text++;
    }
  UART_Write('\r');
  UART_Write('\n');
}

/*====================================================================*/
/*                                                                    */
/*           Sende ein CR/NL                                          */
/*                                                                    */
/*                                                                    */
/*====================================================================*/
void UART_Write_NL()
{
 UART_Write('\r');
 UART_Write('\n');   
}

/*====================================================================*/
/*                                                                    */
/*             Sende einen Integerzahl                                */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

void UART_Write_Int(int zahl)
{
 char string[8];    //Variable string[8] Hier konvertiert itoa später die Zahl hinein
        //itoa(zahl,string, 10);  //Convert Integer zum String, Radix=10  
        while( (UCSRA & 1<<UDRE) == 0 ); // ist Senden möglich, denn Senden
        
        UART_Write_Text(itoa(zahl,string, 10)); //vereinfachen der Ausgabe
}

/*====================================================================*/
/*                                                                    */
/*             Sende eine Integerzahl mit CR/NL                      */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

void UART_Write_Int_NL(int zahl)
{
 char string[8];    //Variable string[8] Hier konvertiert itoa später die Zahl hinein
        //itoa(string,zahl, 10);  //Convert Integer zum String, Radix=10  
       while (!(UCSRA & (1<<UDRE)))  /* warten bis Senden moeglich */
    {
    }
        UART_Write_Text_NL(itoa(zahl,string,  10)); //vereinfachen der Ausgabe
}

/*====================================================================*/
/*                                                                    */
/*              Lese empfangenes Zeichen                              */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

UCHR UART_Read()
{
	 while (!(UCSRA & (1<<RXC)))   // warten bis Zeichen verfuegbar
        ;
    return UDR; 
	return UDR;
}


/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*         #include <stdio.h>  //printf                               */
/*                                                                    */
/*====================================================================*/

static int uart_putchar(char c, FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
static int
uart_putchar(char c, FILE *stream)
{
  if (c == '\n')
    uart_putchar('\r', stream);
  loop_until_bit_is_set(UCSRA, UDRE);
  UDR = c;
  return 0;
}
/*====================================================================*/
/*                                                                    */
/*              Uart-Printf                                           */
/*                                                                    */
/*                                                                    */
/*====================================================================*/

void UART_Printf(char* chr)
{
	 stdout = &mystdout;
	 printf( chr);
}

/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*====================================================================*/
int print_f(char* chr)
{
	 stdout = &mystdout;
	 printf( chr);
 }
 
 
/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*====================================================================*/ 
 
 void serial_echo()
{
	
	
	UCHR chr = UART_Read();
	_delay_ms(100);
	UART_Write(chr);
	_delay_ms(100);
	
}

/* ================================================================== */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/* ================================================================== */

void serial_test()
{
   	//UART_Write_Text_NL("Hallo");
	print_f("Hello Printer\n\r");
	
	}
	
/*====================================================================*/
/*                                                                    */
/*                                                                    */
/*               Main-Routine nur für den Test                        */
/*                                                                    */
/*====================================================================*/
/*
int main(void)
{
	UART_Init();
	
	//UART_Write_Text_NL("Hallo");
	print_f("Hello Printer\n\r");
	return 0;
}
*/
